/*
 * MAX1619/17 device support
 */

#include <epicsStdio.h>
#include <devCommonGpib.h>
#include <recGbl.h>
#include <alarm.h>

/******************************************************************************
 *
 * The following define statements are used to declare the names to be used
 * for the dset tables.   
 *
 * A DSET_AI entry must be declared here and referenced in an application
 * database description file even if the device provides no AI records.
 *
 ******************************************************************************/
#define DSET_AI     devAiMAX1619
#define DSET_LI     devLiMAX1619
#define DSET_LO     devLoMAX1619
#define DSET_BO     devBoMAX1619

#include <devGpib.h> /* must be included after DSET defines */

#define TIMEOUT     5.0    /* I/O must complete within this time */
#define TIMEWINDOW  2.0    /* Wait this long after device timeout */

/*
 * Custom I/O routines
 */
static int
readRegisterValue(struct gpibDpvt *pdpvt, int P1, int P2, char **P3)
{
    struct longinRecord *pli = ((struct longinRecord *)(pdpvt->precord));
    gpibCmd *pgpibCmd = gpibCmdGet(pdpvt);
    asynOctet *pasynOctet = pdpvt->pasynOctet;
    void *asynOctetPvt = pdpvt->asynOctetPvt;
    size_t ntrans;
    int eomReason;
    char reply;

    if ((pasynOctet->write(asynOctetPvt,pdpvt->pasynUser,pgpibCmd->cmd,
                                        1,&ntrans) != asynSuccess)
     || (pasynOctet->read(asynOctetPvt,pdpvt->pasynUser,&reply,
                                        1,&ntrans,&eomReason) != asynSuccess)
     || (ntrans != 1)) {
        return -1;
    }
    if (P1)
        pli->val = (signed char)reply;
    else
        pli->val = (unsigned char)reply;
    pli->udf = 0;
    return 0;
}

static int
createOutputMessage(struct gpibDpvt *pdpvt, int P1, int P2, char **P3)
{
    struct longoutRecord *plo = ((struct longoutRecord *)(pdpvt->precord));
    gpibCmd *pgpibCmd = gpibCmdGet(pdpvt);

    pdpvt->msg[0] = pgpibCmd->format[0];
    pdpvt->msg[1] = plo->val;
    return 2;
}

/******************************************************************************
 *
 * Array of structures that define all GPIB messages
 * supported for this type of instrument.
 *
 ******************************************************************************/

static struct gpibCmd gpibCmds[] = {
    /* Param 0 -- Read manufacturer ID code */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\xFE", NULL, 0, 1,
        readRegisterValue, 0, 0, NULL, NULL, NULL},

    /* Param 1 -- Read Device ID code */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\xFF", NULL, 0, 1,
        readRegisterValue, 0, 0, NULL, NULL, NULL},

    /* Param 2 -- Read local temperature */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x00", NULL, 0, 1,
        readRegisterValue, 1, 0, NULL, NULL, NULL},

    /* Param 3 -- Read remote temperature */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x01", NULL, 0, 1,
        readRegisterValue, 1, 0, NULL, NULL, NULL},

    /* Param 4 -- Read status byte */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x02", NULL, 0, 1,
        readRegisterValue, 0, 0, NULL, NULL, NULL},

    /* Param 5 -- Read configuration byte */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x03", NULL, 0, 1,
        readRegisterValue, 0, 0, NULL, NULL, NULL},

    /* Param 6 -- Read conversion rate byte */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x04", NULL, 0, 1,
        readRegisterValue, 0, 0, NULL, NULL, NULL},

    /* Param 7 -- Read remote Tmax limit */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x10", NULL, 0, 1,
        readRegisterValue, 1, 0, NULL, NULL, NULL},

    /* Param 8 -- Read remote Thyst limit */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x11", NULL, 0, 1,
        readRegisterValue, 1, 0, NULL, NULL, NULL},

    /* Param 9 -- Read remote Thigh limit */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x07", NULL, 0, 1,
        readRegisterValue, 1, 0, NULL, NULL, NULL},

    /* Param 10 -- Read remote Tlow limit */
    {&DSET_LI, GPIBCVTIO, IB_Q_LOW, "\x08", NULL, 0, 1,
        readRegisterValue, 1, 0, NULL, NULL, NULL},

    /* Param 11 -- Write configuration byte */
    {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "\x09", 0, 2,
        createOutputMessage, 0, 0, NULL, NULL, NULL},

    /* Param 12 -- Write conversion rate byte */
    {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "\x0A", 0, 2,
        createOutputMessage, 0, 0, NULL, NULL, NULL},

    /* Param 13 -- Write remote Tmax limit */
    {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "\x12", 0, 2,
        createOutputMessage, 0, 0, NULL, NULL, NULL},

    /* Param 14 -- Write remote Thyst limit */
    {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "\x13", 0, 2,
        createOutputMessage, 0, 0, NULL, NULL, NULL},

    /* Param 15 -- Write remote Thigh limit */
    {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "\x0D", 0, 2,
        createOutputMessage, 0, 0, NULL, NULL, NULL},

    /* Param 16 -- Write remote Tlow limit */
    {&DSET_LO, GPIBWRITE, IB_Q_LOW, NULL, "\x0E", 0, 2,
        createOutputMessage, 0, 0, NULL, NULL, NULL},

    /* Param 17 -- Force conversion (one-shot command) */
    {&DSET_BO, GPIBWRITE, IB_Q_LOW, NULL, "\x0F", 0, 2,
        NULL, 0, 0, NULL, NULL, NULL},

    /* Param 18 -- Write software POR */
    {&DSET_BO, GPIBWRITE, IB_Q_LOW, NULL, "\xFC", 0, 2,
        NULL, 0, 0, NULL, NULL, NULL},

};

/* The following is the number of elements in the command array above.  */
#define NUMPARAMS sizeof(gpibCmds)/sizeof(struct gpibCmd)

/******************************************************************************
 *
 * Initialize device support parameters
 *
 *****************************************************************************/
static long init_ai(int parm)
{
    if(parm==0) {
        devSupParms.name = "devMAX1619";
        devSupParms.gpibCmds = gpibCmds;
        devSupParms.numparams = NUMPARAMS;
        devSupParms.timeout = TIMEOUT;
        devSupParms.timeWindow = TIMEWINDOW;
        devSupParms.respond2Writes = -1;
    }
    return(0);
}
